from localconfig.manager import LocalConfig
import requests

from remoteconfig.utils import url_content


class RemoteConfig(LocalConfig):
  """ A simple wrapper for :class:`localconfig.LocalConfig` to read remote config files with cache option. """

  def __init__(self, last_source=None, cache_duration=None, **localconfig_kwargs):
    """
      :param file/str last_source: Last config source, file or URL. This source is only read when an attempt to read a
                                   config value is made (delayed reading, hence "last") if it exists.
      :param int cache_duration: For URL source only. Optionally cache the URL content for the given duration (seconds) to
                                 avoid downloading too often. This sets the default for all subsequent :meth:`self.read` calls.
      :param dict localconfig_kwargs: Additional keyword args to be passed to localconfig's :meth:`LocalConfig.__init__`
    """

    #: Default cache duration when reading from a URL source
    self._cache_duration = cache_duration

    super(RemoteConfig, self).__init__(last_source, **localconfig_kwargs)

  def read(self, sources, cache_duration=None):
    """
      Queues the config sources to be read later (when config is accessed), or reads immediately if config has already been
      accessed.

      :param file/str/list sources: Config source URL (http/https), source string, file name, or file pointer, or list
                                    of the other sources.  If file source does not exist, it is ignored.
      :param int cache_duration: Default cache durationg for for URL source only. Optionally cache the URL content
                                 for the given duration (seconds) to avoid downloading too often.
                                 This sets the default for all reads now and subsequent reads.
      :return: True if all sources were successfully read or will be read, otherwise False
    """
    if cache_duration is not None:
      self._cache_duration = cache_duration

    return super(RemoteConfig, self).read(sources)

  def _read(self, source):
    """
      Reads and parses the config source

      :param file/str source: Config source URL (http/https), or string, file name, or file pointer.
    """
    if source.startswith('http://') or source.startswith('https://'):
      source = url_content(source, cache_duration=self._cache_duration, from_cache_on_error=True)

    return super(RemoteConfig, self)._read(source)



import os
import time

from mock import patch, Mock
import pytest

from remoteconfig.utils import url_content, _url_content_cache_file


def test_url_content():
  assert 'remoteconfig' in url_content('https://raw.githubusercontent.com/maxzheng/remoteconfig/master/docs/README.rst')

  with patch('requests.get') as requests_get:
    mock_response = Mock()
    mock_response.text = '[Messages]\n1: Test Message'
    requests_get.return_value = mock_response

    assert str(url_content('url1', cache_duration=1)) == mock_response.text

    cached_text = mock_response.text
    mock_response.text = '[Messages]\n1: Test Message Updated'

    # This should return cached content
    assert str(url_content('url1', cache_duration=1)) == cached_text
    requests_get.assert_called_once_with('url1', timeout=5)

    assert str(url_content('url2', cache_duration=1)) == mock_response.text

    time.sleep(1)

    assert str(url_content('url1', cache_duration=1)) == mock_response.text
    assert requests_get.call_count == 3

    cache_file = _url_content_cache_file('url3')

    # Ensure cache file isn't there from previous run
    if os.path.exists(cache_file):
      os.unlink(cache_file)

    requests_get.side_effect = Exception
    with pytest.raises(Exception):
      assert str(url_content('url3', from_cache_on_error=True)) == mock_response.text

    requests_get.side_effect = None
    assert str(url_content('url3', from_cache_on_error=True)) == mock_response.text

    requests_get.side_effect = Exception
    assert str(url_content('url3', from_cache_on_error=True)) == mock_response.text


def test_url_content_cache_file():
  cache_file = _url_content_cache_file('https://raw.githubusercontent.com/maxzheng/remoteconfig/master/config-2.CFG?s=1')
  expected_name = '/url-content-cache-https:__raw.githubusercontent.com_maxzheng_remoteconfig_master_config-2.CFG?s=1'
  assert expected_name == cache_file[-len(expected_name):]



import os
import re
import requests
import tempfile
import time


def _url_content_cache_file(url):
  return os.path.join(tempfile.gettempdir(), 'url-content-cache-%s' % re.sub('[^\w\.:\-=?+]', '_', url[-200:]))


def url_content(url, cache_duration=None, from_cache_on_error=False):
  """
    Get content for the given URL

    :param str url: The URL to get content from
    :param int cache_duration: Optionally cache the content for the given duration to avoid downloading too often.
    :param bool from_cache_on_error: Return cached content on any HTTP request error if available.
  """
  cache_file = _url_content_cache_file(url)

  if cache_duration:
    if os.path.exists(cache_file):
      stat = os.stat(cache_file)
      cached_time = stat.st_mtime
      if time.time() - cached_time < cache_duration:
        with open(cache_file) as fp:
          return fp.read()

  try:
    response = requests.get(url, timeout=5)
    response.raise_for_status()
    content = response.text

  except Exception as e:
    if from_cache_on_error and os.path.exists(cache_file):
      with open(cache_file) as fp:
        return fp.read()
    else:
      raise e.__class__("An error occurred when getting content for %s: %s" % (url, e))

  if cache_duration or from_cache_on_error:
    with open(cache_file, 'w') as fp:
      fp.write(content)

  return content



import time

import httpretty
import pytest
import requests

from remoteconfig import config, RemoteConfig


class TestRemoteConfig(object):
  @httpretty.activate
  def test_read(self):
    config_url = 'http://test-remoteconfig.com/config.ini'

    config_content = '[section]\n\nkey = value\n'
    httpretty.register_uri(httpretty.GET, config_url, body=config_content)
    config.read(config_url)
    assert config_content == str(config)

    # Cache duration on read
    updated_config_content = '[section]\n\nkey = updated\n'
    httpretty.register_uri(httpretty.GET, config_url, body=updated_config_content)
    config.read(config_url, cache_duration=1)
    assert config_content == str(config)

    # Cache duration on init
    config2 = RemoteConfig(config_url, cache_duration=1, kv_sep=': ')
    assert config_content.replace(' = ', ': ') == str(config2)

    # Using default cache duration
    config2.read(config_url)
    assert config_content.replace(' = ', ': ') == str(config2)

    time.sleep(1)

    # Should update after cache duration
    config.read(config_url)
    assert updated_config_content == str(config)

  def test_bad_url(self):
    with pytest.raises(requests.ConnectionError):
      config.read('http://bad-url-that-do-not-exist.bad/config.cfg')
      config.might_get_here_or_not



#!/usr/bin/env python

import os
import setuptools


setuptools.setup(
  name='remoteconfig',
  version='0.2.4',

  author='Max Zheng',
  author_email='maxzheng.os @t gmail.com',

  description=open('README.rst').read(),

  install_requires=[
    'localconfig>=0.4',
    'requests',
  ],

  license='MIT',

  package_dir={'': 'src'},
  packages=setuptools.find_packages('src'),
  include_package_data=True,

  setup_requires=['setuptools-git'],

  classifiers=[
    'Development Status :: 5 - Production/Stable',

    'Intended Audience :: Developers',
    'Topic :: Software Development :: Configuration',

    'License :: OSI Approved :: MIT License',

    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
  ],

  keywords='configuration remote http config',
)
